@@ -12,8 +12,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-__all__ = ["driver", "dynamics", "sampler", "vqs", "TDVP"]
+__all__ = ["driver", "dynamics", "sampler", "vqs", "TDVP", "hilbert", "operator"]
 
+from . import hilbert
+from . import operator
 from . import driver
 from . import dynamics
 from . import sampler
