@@ -153,7 +153,12 @@ class JsonLog(RuntimeLog):
         # Check if the time from the last flush is higher than the maximum
         # allowed runtime cost of flushing
         elapsed_time = time.time() - self._last_flush_time
-        flush_anyway = (self._last_flush_runtime / elapsed_time) < self._autoflush_cost
+        # On windows, the precision of `time.time` is much lower than that on Linux,
+        # so `elapsed_time` may be essentially zero.
+        # We add 1e-7 to avoid the zero division error.
+        flush_anyway = (
+            self._last_flush_runtime / (elapsed_time + 1e-7) < self._autoflush_cost
+        )
 
         if (
             self._steps_notflushed_write % self._write_every == 0
