@@ -19,3 +19,10 @@ def module_version(module: str):
         module = importlib.import_module(module)
 
     return version_tuple(module.__version__)
+
+
+def version_string(module: str):
+    if isinstance(module, str):
+        module = importlib.import_module(module)
+
+    return module.__version__
