@@ -150,7 +150,7 @@ def grad_expect_hermitian_chunked(
     )[0]
 
     Ō_grad = jax.tree_multimap(
-        lambda x, target: (x if jnp.iscomplexobj(target) else x.real).astype(
+        lambda x, target: (x if jnp.iscomplexobj(target) else 2 * x.real).astype(
             target.dtype
         ),
         Ō_grad,
