@@ -35,7 +35,7 @@ def _setup_vmc(dtype=np.float32, sr=True):
 
     # Add custom observable
     X = [[0, 1], [1, 0]]
-    sx = nk.operator.LocalOperator(hi, [X] * L, [[i] for i in range(8)])
+    sx = nk.operator.LocalOperator(hi, [X] * L, [[i] for i in range(L)])
 
     if sr:
         sr_config = nk.optimizer.SR()
@@ -46,44 +46,6 @@ def _setup_vmc(dtype=np.float32, sr=True):
     return ha, sx, vs, sa, driver
 
 
-#####
-@partial(jax.jit, static_argnums=0)
-@partial(jax.vmap, in_axes=(None, None, 0, 0, 0), out_axes=(0))
-def local_value_kernel(logpsi, pars, σ, σp, mel):
-    return jnp.sum(mel * jnp.exp(logpsi(pars, σp) - logpsi(pars, σ)))
-
-
-def local_values(logpsi, variables, Ô, σ):
-    """
-    Returns loc_vals for an operator O
-    """
-    σp, mels = Ô.get_conn_padded(np.asarray(σ).reshape((-1, σ.shape[-1])))
-    loc_vals = local_value_kernel(
-        logpsi, variables, σ.reshape((-1, σ.shape[-1])), σp, mels
-    )
-    return loc_vals.reshape(σ.shape[:-1])
-
-
-def estimate_gradient(logpsi, variables, Ô, σ):
-    """
-    Estimate the gradient of hermmitian operator O using legacy code
-    """
-    O_loc = local_values(logpsi, variables, Ô, σ)
-    O_loc -= O_loc.mean()
-    O_loc /= np.prod(O_loc.shape)
-
-    σr = σ.reshape((-1, σ.shape[-1]))
-    O_loc_r = O_loc.reshape(-1)
-
-    _, vjp = nk.jax.vjp(logpsi, variables, σr, conjugate=True)
-    Opsi = vjp(O_loc_r.conjugate())[0]
-
-    return Opsi
-
-
-####
-
-
 def test_estimate():
     ha, *_, driver = _setup_vmc()
     driver.estimate(ha)
@@ -129,7 +91,8 @@ def test_vmc_functions():
 
     driver.advance(500)
 
-    assert driver.energy.mean == approx(ma.expect(ha).mean, abs=1e-5)
+    tol = driver.energy.error_of_mean * 5
+    assert driver.energy.mean == approx(ma.expect(ha).mean, abs=tol)
 
     state = ma.to_array()
 
@@ -161,7 +124,7 @@ def test_vmc_functions():
         print(mean, var)
 
         # 5-sigma test for expectation values
-        tol = np.sqrt(var / float(ma.n_samples)) * 5
+        tol = op_stats.error_of_mean * 5
         assert mean.real == approx(exact_ex, abs=tol)
 
 
@@ -202,8 +165,8 @@ def central_diff_grad(func, x, eps, *args):
             grad_i = 0.5 * (func(x + 1j * epsd, *args) - func(x - 1j * epsd, *args))
             grad[i] = 0.5 * grad_r + 0.5j * grad_i
         else:
-            grad_i = 0.0
-            grad[i] = 0.5 * grad_r
+            # grad_i = 0.0
+            grad[i] = grad_r
 
         assert not np.isnan(grad[i])
         grad[i] /= eps
@@ -229,16 +192,6 @@ def test_vmc_gradient(dtype):
     ha, sx, ma, sampler, driver = _setup_vmc(dtype=dtype, sr=False)
     driver.run(3, out=None)
 
-    # old formula
-    _, grads = ma.expect_and_grad(ha)
-    grad_old = estimate_gradient(
-        lambda w, x: ma._apply_fun({"params": w}, x), ma.parameters, ha, ma.samples
-    )
-
-    # check that the old fformula and the new give same result
-    jax.tree_multimap(lambda x, y: same_derivatives(x, y), grads, grad_old)
-    # end old
-
     pars_0 = ma.parameters
     pars, unravel = nk.jax.tree_ravel(pars_0)
 
