@@ -17,6 +17,7 @@ import pytest
 from functools import partial
 
 import jax
+import jax.numpy as jnp
 import numpy as np
 from numpy import testing
 import jax.flatten_util
@@ -26,6 +27,9 @@ import netket as nk
 import netket.jax as nkjax
 from netket.optimizer import qgt
 
+from netket.optimizer.qgt.qgt_jacobian_pytree import QGTJacobianPyTreeT
+from netket.optimizer.qgt.qgt_jacobian_dense import QGTJacobianDenseT
+
 from .. import common
 
 QGT_objects = {}
@@ -138,7 +142,23 @@ def test_qgt_solve(qgt, vstate, solver, _mpi_size, _mpi_rank):
             S = qgt(vstate)
             x_all, _ = S.solve(solver, vstate.parameters)
 
-            jax.tree_multimap(lambda a, b: np.testing.assert_allclose(a, b), x, x_all)
+            jax.tree_multimap(
+                lambda a, b: np.testing.assert_allclose(a, b, rtol=0.00045), x, x_all
+            )
+
+
+@pytest.mark.skipif_mpi
+@pytest.mark.parametrize(
+    "qgt",
+    [pytest.param(sr, id=name) for name, sr in QGT_objects.items()],
+)
+@pytest.mark.parametrize("chunk_size", [None])
+def test_qgt_solve_with_x0(qgt, vstate):
+    solver = jax.scipy.sparse.linalg.gmres
+    x0 = jax.tree_map(jnp.zeros_like, vstate.parameters)
+
+    S = qgt(vstate)
+    x, _ = S.solve(solver, vstate.parameters, x0=x0)
 
 
 @pytest.mark.parametrize(
@@ -225,3 +245,31 @@ def test_qgt_dense(qgt, vstate, _mpi_size, _mpi_rank):
             Sd_all = S.to_dense()
 
             np.testing.assert_allclose(Sd_all, Sd, rtol=1e-5, atol=1e-15)
+
+
+@pytest.mark.skipif_mpi
+@pytest.mark.parametrize(
+    "qgt", [pytest.param(sr, id=name) for name, sr in QGT_objects.items()]
+)
+@pytest.mark.parametrize(
+    "chunk_size",
+    [
+        None,
+    ],
+)
+def test_qgt_pytree_diag_shift(qgt, vstate):
+    v = vstate.parameters
+    S = qgt(vstate)
+    expected = S @ v
+    diag_shift = S.diag_shift
+    if isinstance(S, (QGTJacobianPyTreeT, QGTJacobianDenseT)):
+        # extract the necessary shape for the diag_shift
+        t = jax.eval_shape(partial(jax.tree_map, lambda x: x[0], S.O))
+    else:
+        t = v
+    diag_shift_tree = jax.tree_map(
+        lambda x: diag_shift * jnp.ones(x.shape, dtype=x.dtype), t
+    )
+    S = S.replace(diag_shift=diag_shift_tree)
+    res = S @ v
+    jax.tree_multimap(lambda a, b: np.testing.assert_allclose(a, b), res, expected)
